﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Parsing;
using OpcLabs.DotProlog.Terms;

namespace DotPrologTest.Parsing
{
    [TestClass]
    public class ReadTermParserTest : ReadTermParserTestBase
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void TryParseTest_TextReader_AncestorClause2()
        {
            var X = new Variable();
            var Y = new Variable();
            var Z = new Variable();
            TestSuccess(
                "ancestor(X,Y) :- parent(X,Y), ancestor(Y,Z).",
                new [] { "X", "Y", "Z"},
                new NTerm(":-")
                {
                    new NTerm("ancestor") {X, Y},
                    new NTerm(",") {new NTerm("parent") {X, Y}, new NTerm("ancestor") {Y,Z}},
                });
        }

        [TestMethod]
        public void TryParseTest_TextReader_AppendClause1()
        {
            var X = new Variable();
            TestSuccess(
                "append([],X,X).",
                new [] { "X" },
                new NTerm("append") { "[]", X, X });
        }

        [TestMethod]
        public void TryParseTest_TextReader_AppendClause2()
        {
            var Q1 = new Variable();
            var Q2 = new Variable();
            var T = new Variable();
            var X = new Variable();
            TestSuccess(
                "append([T|Q1],X,[T|Q2]) :- append(Q1,X,Q2).",
                new [] { "X", "T", "Q1", "Q2"},
                new NTerm(":-")
                {
                    new NTerm("append") {new NList(new[]{T}, Q1), X, new NList(new[]{T}, Q2)},
                    new NTerm("append") {Q1, X, Q2},
                });
        }

        [TestMethod]
        public void TryParseTest_TextReader_EmptyList()
        {
            TestSuccess("[].", new NTerm("[]") );
        }

        [TestMethod]
        public void TryParseTest_TextReader_FunctorNotation()
        {
            TestSuccess(
                "animal(cat).",
                new NTerm("animal") { "cat" });
        }

        [TestMethod]
        public void TryParseTest_UnexpectedPrefixOperator()
        {
            TestError(@"a ?-", TermSyntaxErrorCode.UnexpectedPrefixOperator);
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
