﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using OpcLabs.BaseLib.Forms.Connectivity;
using OpcLabs.BaseLib.Forms.Extensions;
using OpcLabs.BaseLib.Forms.Testing;
using OpcLabs.BaseLib.Persistence;
using OpcLabs.EasyOpc.AlarmsAndEvents;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.Connectivity;
using OpcLabs.EasyOpc.DataAccess.Reactive;
using OpcLabs.EasyOpc.Forms.Browsing;
using OpcLabs.EasyOpc.UA.Connectivity;

namespace OpcLabs.EasyOpcFormsTestApp.DataAccess
{
    static class DACommands
    {

        static internal void BrowseControls()
        {
            var form = new DABrowseControlsForm();
            form.Show();
        }

        static internal void LiveBinding()
        {
            var form = new DALiveBindingForm();
            form.Show();
        }

        static internal void ObjectTestbed()
        {
            var objectTypes = new List<Type>();
            objectTypes.AddRange(
                typeof(DAQuality).Assembly.GetTypes()
                .Concat(typeof(EasyDAClient).Assembly.GetTypes())
                .Concat(typeof(EasyDAClientManagement).Assembly.GetTypes())
                .Concat(typeof(DAItemChangedObservable).Assembly.GetTypes())
                .Concat(typeof(OpcBrowseDialog).Assembly.GetTypes()));
            var favoriteObjectTypes = new List<Type>
                {
                    typeof (EasyDAClient),
                    typeof (EasyAEClient)
                };
            Type primaryType = typeof(EasyDAClient);

            var form = new ObjectTestbedForm();
            form.SetOnlyUsableTypes(objectTypes);
            form.FavoriteTypes = favoriteObjectTypes;
            form.CurrentType = primaryType;
            var deferredInstanceInjector = new DeferredInstanceInjector(form.Instances);
            // ...
            form.Instances = deferredInstanceInjector;
            form.InitialAction = ObjectTestbedFormAction.Instantiate;
            form.ImportedNamespaces = new[]
                {
                    "OpcLabs.BaseLib.Diagnostics",
                    "OpcLabs.EasyOpc.DataAccess",
                    "OpcLabs.EasyOpc.AlarmsAndEvents",
                    "System",
                    "System.Collections.Generic"
                };

            form.Show();
        }

        static internal void PointEditorControlWithDAConnectivity()
        {
            var control = new PointEditorControl { Connectivity = new DAConnectivity() };
            control.ShowInDialog(null, "Dialog made of Point Editor Control", new []{DialogResult.OK}, DialogResult.OK);
        }

        static internal void PointEditorControlWithUAConnectivity()
        {
            var control = new PointEditorControl { Connectivity = new UAConnectivity() };
            control.ShowInDialog(null, "Dialog made of Point Editor Control", new[] { DialogResult.OK }, DialogResult.OK);
        }
    }
}
