﻿
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.DataTypeModel;

namespace OpcLabs.EasyOpcTest.UA.DataTypeModel.Internal
{
    partial class TypeDescriptionExamplesTest
    {
        #region Static Private Properties

        [NotNull]
        static private EnumerationDataType TrafficLightDataType
        {
            get
            {
                var trafficLightDataType = new EnumerationDataType("TrafficLight", "TypeDescriptionExamples:TrafficLight", 32)
                    {
                        new EnumerationMember("Red", 4) {Description = "Red says stop immediately."},
                        new EnumerationMember("Yellow", 3) {Description = "Yellow says prepare to stop."},
                        new EnumerationMember("Green", 2) {Description = "Green says you may proceed."}
                    };
                trafficLightDataType.Description = "The possible colours for a traffic signal.";
                return trafficLightDataType;
            }
        }

        [NotNull]
        static private EnumerationData TrafficLight_0Data => new EnumerationData(TrafficLightDataType, 0);

        [NotNull]
        static private byte[] TrafficLight_0ExtensionObjectBody => new byte[4];

        [NotNull]
        static private EnumerationData TrafficLight_2Data => new EnumerationData(TrafficLightDataType, 2);

        [NotNull]
        static private byte[] TrafficLight_2ExtensionObjectBody => new byte[] { 2, 0, 0, 0 };

        [NotNull]
        static private EnumerationData TrafficLight_4Data => new EnumerationData(TrafficLightDataType, 4);

        [NotNull]
        static private byte[] TrafficLight_4ExtensionObjectBody => new byte[] { 4, 0, 0, 0 };

        #endregion



        #region Test Methods

        [TestMethod]
        public void DecodeTest_TrafficLight_0() =>
            DecodeTest(null, TypeDictionaryByteArray, "TrafficLight", TrafficLight_0ExtensionObjectBody, TrafficLight_0Data);

        [TestMethod]
        public void DecodeTest_TrafficLight_2() =>
            DecodeTest(null, TypeDictionaryByteArray, "TrafficLight", TrafficLight_2ExtensionObjectBody, TrafficLight_2Data);

        [TestMethod]
        public void DecodeTest_TrafficLight_4() =>
            DecodeTest(null, TypeDictionaryByteArray, "TrafficLight", TrafficLight_4ExtensionObjectBody, TrafficLight_4Data);

        [TestMethod]
        public void EncodeTest_TrafficLight_0() =>
            EncodeTest(null, TypeDictionaryByteArray, "TrafficLight", TrafficLight_0Data, TrafficLight_0ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_TrafficLight_2() =>
            EncodeTest(null, TypeDictionaryByteArray, "TrafficLight", TrafficLight_2Data, TrafficLight_2ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_TrafficLight_4() =>
            EncodeTest(null, TypeDictionaryByteArray, "TrafficLight", TrafficLight_4Data, TrafficLight_4ExtensionObjectBody);

        #endregion
    }
}
