﻿
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Threading;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;

namespace OpcLabs.OpcInteroperabilityTest.UA.Servers
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    // ReSharper disable IdentifierTypo
    public class WagoKontaktTechnik_ControllersPfc100AndPfc200_Test : UAInteroperabilityWorkshopTest
        // ReSharper restore IdentifierTypo
    {
        private const string DynamicNode1String = "ns=4;s=|var|plc1.advanced.Main.Main_real";
        private const string DynamicNode2String = "ns=4;s=|var|plc1.advanced.Main.Main_lreal";


        class DriveDynamicNodesBackgroundWorker : BackgroundWorker
        {
            public DriveDynamicNodesBackgroundWorker(
                [NotNull] IEasyUAClient client,
                [NotNull] UAEndpointDescriptor endpointDescriptor)
            {
                _client = client;
                _endpointDescriptor = endpointDescriptor;
                WorkerSupportsCancellation = true;

                DoWork += OnDoWork;
            }

            private void OnDoWork(object sender, [NotNull] DoWorkEventArgs e)
            {
                float dynamicValue1 = 0.0f;
                float dynamicValue2 = 0.0f;

                do
                {
                    if (CancellationPending)
                    {
                        e.Cancel = true;
                        return;
                    }

                    _client.WriteValue(_endpointDescriptor, DynamicNode1String, dynamicValue1);
                    _client.WriteValue(_endpointDescriptor, DynamicNode2String, dynamicValue2);

                    dynamicValue1 += 0.1f;
                    dynamicValue2 += 0.2f;

                    Thread.Sleep(100);
                } while (true);
            }

            [NotNull]
            private readonly IEasyUAClient _client;

            [NotNull] 
            private readonly UAEndpointDescriptor _endpointDescriptor;
        }
        


        public override IEnumerable<UANodeDescriptor> ArrayReadableNodes => new UANodeDescriptor[]
        {
            "ns=4;s=|var|plc1.advanced.Arr_Dim1.arrayBYTE",
            "ns=4;s=|var|plc1.advanced.Arr_Dim1.arrayREAL",
            "ns=4;s=|var|plc1.advanced.Arr_Dim2.arrayBYTE"  //2-dimensional
        };

        public override IEnumerable<UANodeDescriptor> ArrayWriteableNodes => new UANodeDescriptor[]
        {
            "ns=4;s=|var|plc1.advanced.Arr_Dim1.arrayBYTE",
            "ns=4;s=|var|plc1.advanced.Arr_Dim1.arrayREAL"
        };

        public override IEnumerable<UANodeDescriptor> DynamicNodes => new UANodeDescriptor[]
        {
            DynamicNode1String,
            DynamicNode2String
        };

        public override IEnumerable<UAEndpointDescriptor> EndpointDescriptors => 
            new UAEndpointDescriptor[] { "opc.tcp://WagoPFC1.european.iop.virtual.opcfoundation.org:4840" };

        public override IEnumerable<string> ServerNotSupportedTests => base.ServerNotSupportedTests.Concat(new []
        {
            //nameof(Group001_Session_Test02_CreateSession_UaTcp_Basic128Rsa15),
            //nameof(Group001_Session_Test08_CreateSession_InstanceCertificate),
            //nameof(Group001_Session_Test09_CreateSession_Https_None),
            nameof(Group001_Session_Test12_CreateSession_UaTcp_Aes128_Sha256_RsaOaep),
            nameof(Group001_Session_Test13_CreateSession_UaTcp_Aes256_Sha256_RsaPss),
            nameof(Group004_MethodServices_Test01_Call),
            nameof(Group008_Eventing_Test01_Subscribe_Server),
            nameof(Group011_AlarmCondition_Test01_Browse_Types),
            nameof(Group102_Session2_Test01_CreateSession_UaTcp_Basic256),
            nameof(Group102_Session2_Test03_CreateSession_Https_Basic256Sha256)
        });

        // ReSharper disable once StringLiteralTypo
        public override string Password => "wago";

        public override IEnumerable<UANodeDescriptor> StaticNodes => new UANodeDescriptor[]
        {
            "ns=4;s=|var|plc1.advanced.GVL.global_byte",
            "ns=4;s=|var|plc1.advanced.GVL.global_real"
        };

        public override string UserName => "admin";

        public override IEnumerable<UANodeDescriptor> WriteableNodes => new UANodeDescriptor[]
        {
            "ns=4;s=|var|plc1.advanced.GVL.global_byte",
            "ns=4;s=|var|plc1.advanced.GVL.global_real"
        };


        protected override BackgroundWorker CreateDriveDynamicNodesBackgroundWorker(IEasyUAClient client) =>
            new DriveDynamicNodesBackgroundWorker(client, EndpointDescriptors.First());
    }
}
